package nccloud.zjnytwo_cd.zjnytwo_cd.bankcredithvo.action;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;

import nc.bs.dao.BaseDAO;
import nc.bs.dao.DAOException;
import nc.bs.framework.common.NCLocator;
import nc.bs.logging.Logger;
import nc.jdbc.framework.processor.ColumnProcessor;
import nc.md.model.MetaDataException;
import nc.md.persist.framework.IMDPersistenceQueryService;
import nc.vo.pub.BusinessException;
import nc.vo.pub.lang.UFDate;
import nc.vo.pub.lang.UFDouble;
import nc.vo.zjnytwo_cd.bankcredit.BankcreditBVO;
import nc.vo.zjnytwo_cd.bankcredit.BankcreditBillVO;
import nc.vo.zjnytwo_cd.bankcredit.BankcreditHVO;
import nccloud.framework.core.exception.ExceptionUtils;
import nccloud.framework.core.io.WebFile;
import nccloud.framework.web.action.itf.ICommonAction;
import nccloud.framework.web.container.IRequest;
import nccloud.framework.web.container.SessionContext;
import nccloud.pubitf.riart.pflow.CloudPFlowContext;
import nccloud.pubitf.riart.pflow.ICloudScriptPFlowService;
import nccloud.vo.zjnytwo_cd.zjnytwo_cd.BankcreditHVOConst;
import nccloud.zjnytwo_cd.zjnytwo_cd.bankcredithvo.zjnytwo_cd.util.CommonUtil;
import nccloud.zjnytwo_cd.zjnytwo_cd.tool.Excel;

/**
 * 银行授信填报导入
 * @author LP
 *
 */
public class BankcreditHVOImportAction implements ICommonAction {
	
	public String group;
	public String org;
	public String org_v;
	public int month;
	public String year;
	
	public BaseDAO dao;
	public IMDPersistenceQueryService qryService;

	@Override
	public Object doAction(IRequest paramIRequest) {
		
		WebFile[] readFiles = paramIRequest.readFiles();
		
		WebFile file = readFiles[0];

		try {
			
			dao = new BaseDAO();
			qryService = NCLocator.getInstance().lookup(IMDPersistenceQueryService.class);
			
			JSONArray jsonArray = Excel.readXLSX(file);//解析Excel
			
			BankcreditBillVO bankcreditBillVO = setBankcreditBillVO(jsonArray);
			
			saveBankcreditBillVO(bankcreditBillVO);
			
		} catch (Exception e) {
			Logger.error(e);
		    ExceptionUtils.wrapException(e);
		}
		
		return null;
	}
	
	/*
	 * 保存VO
	 */
	private void saveBankcreditBillVO(BankcreditBillVO bankcreditBillVO) throws BusinessException {
		
		uniform(bankcreditBillVO);
		
		judgment(bankcreditBillVO);
		
		BankcreditBillVO bVo = showBankcreditBillVOs();
		
		if (bVo!=null) {
			deleteVO(bVo);//删除VO
		}
		saveVO(bankcreditBillVO);
	}
	
	/**
	   * 表头表体单位一致校验
	   * @param vo
	   * @throws DAOException
	   */
	  public void uniform(BankcreditBillVO vo) throws DAOException {
		
		  String headorg = vo.getParentVO().getPk_org();
		  String bodyorg = "";
		  
		  BankcreditBVO[] bvos = (BankcreditBVO[])vo.getChildrenVO();
		  
		  for (BankcreditBVO bankcreditBVO : bvos) {
			  
			  bodyorg = bankcreditBVO.getUnit();
			
			  if (!bodyorg.equals(headorg)) {
				throw new DAOException("表头一级组织与表体一级单位必须一致！");
			  }
			  
		  }
		  
	  }
	
	/*
	   * 校验单位映射是否正确
	   */
	  public void judgment(BankcreditBillVO bankcreditBillVO) throws DAOException {
			
		String unit = "";
		String credit = "";
		String unitfather = "";
		String creditunitfather = "";
		
		BankcreditBVO[] childrenVO = (BankcreditBVO[]) bankcreditBillVO.getChildrenVO();
		
		String father = SqlQuery("pk_financeorg", "org_financeorg", "code", "10", "一级财务组织编码10");
		
		String org = bankcreditBillVO.getParentVO().getPk_org();
		String orgfather = SqlQuery("pk_fatherorg", "org_orgs", "pk_org", org, "组织父单位");
		String orgname = SqlQuery("name", "org_orgs", "pk_org", org, "一级组织");
		
		if (orgfather!=null && !orgfather.equals(father)) {
			throw new DAOException("一级组织'"+orgname+"'不是一级组织，请检查数据的合法性！");
		}else if (orgname.equals("全局")) {
			throw new DAOException("一级组织不能为空，如是'全局'请修改！");
		}
			
		for (int i = 0; i < childrenVO.length; i++) {
				
			BankcreditBVO bvo = childrenVO[i];
				
			unit = bvo.getUnit();
			credit = bvo.getCreditunit();
			
			String unitName = SqlQuery("name", "org_financeorg", "pk_financeorg", unit, "一级单位");
			String creditName = SqlQuery("name", "org_financeorg", "pk_financeorg", credit, "授信单位");
			
			unitfather = SqlQuery("pk_fatherorg", "org_orgs", "pk_org", unit, "一级父单位");
			
			if (unitfather!=null && !unitfather.equals(father)) {
				throw new DAOException("一级单位'"+unitName+"'不是一级组织，请检查数据的合法性！");
			}else if (credit.equals(father)&&!unit.equals(credit)) {
				throw new DAOException("授信单位'"+creditName+"'不是一级单位'"+unitName+"'的子公司，不能保存！");
			}else if(unit.equals(credit)) {
				return;
			}
			
			creditunitfather = SqlQuery("pk_fatherorg", "org_orgs", "pk_org", credit, "授信父单位");
			
			for (int k = 0; k < 10; k++) {
				
				if (creditunitfather.equals(unit)) {
					break;
				}else if (creditunitfather.equals(father)) {
					throw new DAOException("授信单位'"+creditName+"'不是一级单位'"+unitName+"'的子公司，不能保存！");
				}else{
					creditunitfather=SqlQuery("pk_fatherorg", "org_orgs", "pk_org", creditunitfather, "授信父单位");
				}
				
				
			}
		}
	  }
	
	/*
	 * 查找VO
	 */
	private BankcreditBillVO showBankcreditBillVOs() throws MetaDataException {
		
		String sql = "month = '"+String.valueOf(month)+"' and pk_org = '"+org+"' and dr = '0' and def1 = '"+year+"'";
		
		BankcreditBillVO oldBankcreditBillVO = getVO(sql);
		
		if (oldBankcreditBillVO==null) {
			return null;
		}
		
		return oldBankcreditBillVO;
	}
	
	/*
	 * 组合BankcreditBillVO
	 */
	private BankcreditBillVO setBankcreditBillVO(JSONArray jsonArray) throws Exception {
		
		BankcreditBillVO bankcreditBillVO = new BankcreditBillVO();
		
		JSONArray headArray = jsonArray.getJSONArray(0);//表头
		JSONArray bodyArray = jsonArray.getJSONArray(1);//表体
		
		BankcreditHVO headHvo = sethead(headArray);
		BankcreditBVO[] bodyBvos = setbody(bodyArray);
		
		bankcreditBillVO.setParentVO(headHvo);
		bankcreditBillVO.setChildrenVO(bodyBvos);
		
		return bankcreditBillVO;
	}
	
	/*
	 * 导入表头
	 */
	@SuppressWarnings("static-access")
	private BankcreditHVO sethead(JSONArray headArray) throws Exception {
		
		BankcreditHVO bankcreditHVO = new BankcreditHVO();
		JSONObject headObject = headArray.getJSONObject(0);
		
		String enablestate = headObject.getString("启用");
		
		if (headArray.size()>1) {
			throw new Exception("表头不允许有多列！请检查数据的合法性或Excel表格是否符合规范！");
		}
		
		if (enablestate==null) {
			throw new Exception("启用为必填项，请检查！");
		}
		
		if (headObject.getString("组织")==null) {
			throw new Exception("组织为必填项，请检查！");
		}
		
		if(enablestate.equals("Y")) {
			bankcreditHVO.setEnablestate(2);
		}else if(enablestate.equals("N")) {
			bankcreditHVO.setEnablestate(3);
		}else {
			throw new Exception("启用只允许填写'N'和'Y'，请检查！");
		}
		
		group = SqlQuery("pk_group", "org_group", "name", headObject.getString("集团"),"集团");
		org = SqlQuery("pk_financeorg", "org_financeorg", "name", headObject.getString("组织"),"组织");
		org_v = SqlQuery("pk_vid", "org_financeorg_v", "name", headObject.getString("组织"),"组织");
		
		bankcreditHVO.setPk_group(group);
		bankcreditHVO.setPk_org(org);
		bankcreditHVO.setPk_org_v(org_v);
		
		month = Integer.valueOf(headObject.getString("月份"));
		bankcreditHVO.setMonth(month);//月份
		
		if (month<1 || month>12) {
			throw new BusinessException("月份只允许填写1~12的数字！");
		}
		
		Date date = new Date();
		SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd"); 
		String Inputdate = formatter.format(date);
		bankcreditHVO.setInputdate(new UFDate(Inputdate));//导入日期
		bankcreditHVO.setDbilldate(new UFDate(Inputdate));//单据日期
		
		String cuserid = SessionContext.getInstance().getClientInfo().getUserid();
		
		bankcreditHVO.setInputman(cuserid);
		bankcreditHVO.setCode(headObject.getString("编码"));
		bankcreditHVO.setName(headObject.getString("名称"));
		bankcreditHVO.setVnote(headObject.getString("备注"));
		
		if(headObject.getString("年份").equals("")) {
			SimpleDateFormat SimpleDate = new SimpleDateFormat("yyyy");
	        year = SimpleDate.format(date);
	        bankcreditHVO.setDef1(year);
		}else {
			if (this.isNumber(headObject.getString("年份"))) {
				year = headObject.getString("年份");
				bankcreditHVO.setDef1(year);
			}
		}
		
		return bankcreditHVO;
	}
	
	/**
	 * 判断年份
	 * @param str
	 * @return
	 * @throws Exception
	 */
	private static boolean isNumber(String str) throws Exception{
        String reg = "^[0-9]+(.[0-9]+)?$";
        if(str.matches(reg)) {
        	if (str.length()>4 || str.length()<4) {
				throw new Exception("请输入正确的年份！");
			}else {
				return true;
			}
        }else {
        	throw new Exception("年份只允许填入数字，请检查！");
		}
    }
	
	/*
	 * 导入表体
	 */
	private BankcreditBVO[] setbody(JSONArray bodyArray) throws DAOException {
		
		BankcreditBVO[] bvos = new BankcreditBVO[bodyArray.size()];
		
		for(int i = 0 ; i < bodyArray.size() ; i++ ) {
			
			JSONObject bodyobject = bodyArray.getJSONObject(i);
			
			BankcreditBVO bankcreditBVO = new BankcreditBVO();
			
			bankcreditBVO.setPk_group(group);
			bankcreditBVO.setPk_org(org);
			bankcreditBVO.setPk_org_v(org_v);
			bankcreditBVO.setCrowno(bodyobject.getString("行号"));
			bankcreditBVO.setUnit(SqlQuery("pk_financeorg", "org_financeorg", "name", bodyobject.getString("一级单位"),"一级单位"));
			bankcreditBVO.setCreditunit(SqlQuery("pk_financeorg", "org_financeorg", "name", bodyobject.getString("授信单位"),"授信单位"));
			bankcreditBVO.setBanktype_code(bodyobject.getString("银行类别编码"));
			bankcreditBVO.setBanktype(bodyobject.getString("银行类别"));
			bankcreditBVO.setCurrency(bodyobject.getString("币种"));
			bankcreditBVO.setAmount(new UFDouble(bodyobject.getString("有效授信金额")));
			bankcreditBVO.setUsedamount(new UFDouble(bodyobject.getString("已使用金额")));
			bankcreditBVO.setRemainderamount(new UFDouble(bodyobject.getString("有效剩余金额")));
			bankcreditBVO.setAmountloan(new UFDouble(bodyobject.getString("已使用金额_贷款")));
			bankcreditBVO.setAmountcredit(new UFDouble(bodyobject.getString("已使用金额_信用证")));
			bankcreditBVO.setAmountdraft(new UFDouble(bodyobject.getString("已使用金额_承兑汇票")));
			bankcreditBVO.setAmountoccupied(new UFDouble(bodyobject.getString("已使用金额_保函占用")));
			bankcreditBVO.setVbnote(bodyobject.getString("行备注"));
			
			bvos[i] = bankcreditBVO;
		}
		
		return bvos;
	}
	
	/*
	 * VO查询方法
	 */
	@SuppressWarnings("unchecked")
	private BankcreditBillVO getVO(String sql) throws MetaDataException {
		ArrayList<BankcreditBillVO> AggVOs = (ArrayList<BankcreditBillVO>) qryService.queryBillOfVOByCond(BankcreditBillVO.class,sql,true,false);
		
		if (AggVOs.size()==0) {
			return null;
		}
		
		return AggVOs.get(0);
	}
	
	/*
	 * SQL查询方法
	 */
	private String SqlQuery(String out,String table,String lock,String key,String remark) throws DAOException {
		
		if (key==null || key.equals("")) {
			return null;
		}
		
		String sql = "select "+out+" from "+table+" where "+lock+" = '"+key+"' and dr = '0'";
		
		Object object = dao.executeQuery(sql, new ColumnProcessor());
		
		if (remark.equals("一级父单位") && object == null) {
			return null;
		}else if (remark.equals("组织父单位") && object == null) {
			return null;
		}else if (object == null) {
			throw new DAOException("根据"+remark+"值"+key+"在数据库中未查到相应的数据，请检查！");
		}
		
		return object.toString();
	}
	
	/*
	 * 保存脚本
	 */
	@SuppressWarnings("unchecked")
	private BankcreditBillVO saveVO(BankcreditBillVO... aggVOs)
		      throws BusinessException {

		    String actionCode = BankcreditHVOConst.CONST_ACTION_SAVEBASE;
		    String billType = BankcreditHVOConst.CONST_BILLTYPE_COST;

		    CloudPFlowContext context = new CloudPFlowContext();
		    context.setActionName(actionCode);
		    context.setBillType(billType);
		    context.setBillVos(aggVOs);
		    Logger.debug("开始调用动作脚本 ActionName[" + actionCode + "] BillType[" + billType
		        + "]...");

		    ICloudScriptPFlowService service =
		        CommonUtil.getCloudScriptPFlowService();

		    Object[] result = service.exeScriptPFlow(context);

		    Logger.debug("调用动作脚本 ActionName[" + actionCode + "] BillType[" + billType
		        + "]结束");

		    String wheresql =
		        "pk_bankcredit" + "='"
		            + ((BankcreditBillVO) result[0]).getPrimaryKey() + "'";
		    Collection<BankcreditBillVO> bills =
		        CommonUtil.getMDPersistenceQueryService().queryBillOfVOByCond(
		            BankcreditBillVO.class, wheresql, true, false);
		    return bills.toArray(new BankcreditBillVO[0])[0];
	}
	
	/*
	 * 删除脚本
	 */
	private Object deleteVO(BankcreditBillVO... aggVOs)
		      throws BusinessException {

		    if ((aggVOs == null) || (aggVOs.length == 0)) {
		      return null;
		    }

		    String actionCode = BankcreditHVOConst.CONST_ACTION_DELETE;
		    String billType = BankcreditHVOConst.CONST_BILLTYPE_COST;

		    CloudPFlowContext context = new CloudPFlowContext();
		    context.setActionName(actionCode);
		    context.setBillType(billType);
		    context.setBillVos(aggVOs);
		    Logger.debug("开始调用动作脚本 ActionName[" + actionCode + "] BillType[" + billType
		        + "]...");

		    ICloudScriptPFlowService service = CommonUtil.getCloudScriptPFlowService();
		    Object[] result = service.exeScriptPFlow(context);

		    Logger.debug(
		        "调用动作脚本 ActionName[" + actionCode + "] BillType[" + billType + "]结束");

		    return result;
	}

}
