
package nccloud.zjnytwo_cd.zjnytwo_cd.bankcredithvo.action;

import java.text.SimpleDateFormat;
import java.time.Year;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;

import nc.bs.dao.BaseDAO;
import nc.bs.dao.DAOException;
import nc.bs.framework.common.NCLocator;
import nc.bs.logging.Logger;
import nc.jdbc.framework.processor.ColumnProcessor;
import nc.md.model.MetaDataException;
import nc.md.persist.framework.IMDPersistenceQueryService;
import nccloud.vo.zjnytwo_cd.zjnytwo_cd.BankcreditHVOConst;
import nc.vo.zjnytwo_cd.bankaccounts.BankaccountsBVO;
import nc.vo.zjnytwo_cd.bankaccounts.BankaccountsBillVO;
import nc.vo.zjnytwo_cd.bankcredit.BankcreditBVO;
import nc.vo.zjnytwo_cd.bankcredit.BankcreditBillVO;
import nc.vo.zjnytwo_cd.bankcredit.BankcreditHVO;
import nc.vo.pub.BusinessException;
import nc.vo.pub.CircularlyAccessibleValueObject;
import nccloud.framework.core.exception.ExceptionUtils;
import nccloud.framework.service.ServiceLocator;
import nccloud.framework.web.action.itf.ICommonAction;
import nccloud.framework.web.container.IRequest;
import nccloud.framework.web.ui.model.row.Cell;
import nccloud.framework.web.ui.model.row.Row;
import nccloud.framework.web.ui.pattern.billcard.BillCard;
import nccloud.framework.web.ui.pattern.billcard.BillCardOperator;
import nccloud.pubitf.riart.pflow.CloudPFlowContext;
import nccloud.pubitf.riart.pflow.ICloudScriptPFlowService;
import nccloud.zjnytwo_cd.zjnytwo_cd.bankcredithvo.zjnytwo_cd.util.CommonUtil;

/**
 * 主子表保存
 * @version  @since v3.5.6-1903
 */
public class BankcreditHVOSaveAction implements ICommonAction {

	String deletePk = "";
	String year = "";
	
  @Override
  public Object doAction(IRequest paramIRequest) {
    try {
    	
      Date date = new Date();
  	  SimpleDateFormat SimpleDate = new SimpleDateFormat("yyyy");
      year = SimpleDate.format(date);
    	
      BillCardOperator billCardOperator = new BillCardOperator();
      // 1、获取AGGVO （request转换主子VO）
      BankcreditBillVO vo = billCardOperator.toBill(paramIRequest);
      this.doBefore(vo);
      //#表头表体单位一致校验
      uniform(vo);
      //#校验单位映射是否正确
      judgment(vo);
      //#判断月份
      judgmentmonth(vo);
      //#添加年份
      String parentPk = vo.getPrimaryKey();
      if ((parentPk == null) || "".equals(parentPk)) {
    	  vo.getParentVO().setDef1(year);
      }
      // 2、调用单据的保存动作脚本（savebase），得到保存后结果
      BankcreditBillVO rtnObj = this.callActionScript(vo);
      // 3、处理返回结果（包含功能：根据模板转换前端BillCard，参照翻译，显示公式处理）
	  BillCard billcard = billCardOperator.toCard(rtnObj);
      //#返回被删除数据的PK值
      if (!deletePk.equals("")) {
    	  Cell cell = new Cell();
    	  cell.setValue(deletePk);
    	  billcard.getHead().getModel().getListRows().get(0).addCell("deletepk", cell);
      }
      deletePk = "";
      // 4、返回结果到前端
      return billcard;
    }
    catch (BusinessException ex) {
      // 处理异常信息
      Logger.error(ex);
      ExceptionUtils.wrapException(ex);
    }
    return null;
  }

  /**
   * 动作编码
   *
   * @return
   */
  protected String getActionCode() {
    return BankcreditHVOConst.CONST_ACTION_SAVEBASE;
  }

  /**
   * 调用动作脚本
   *
   * @param actionCode
   * @param aggVOs
   * @return
   * @throws BusinessException
   */
  private BankcreditBillVO callActionScript(BankcreditBillVO... aggVOs)
      throws BusinessException {

    String actionCode = this.getActionCode();
    String billType = BankcreditHVOConst.CONST_BILLTYPE_COST;

    CloudPFlowContext context = new CloudPFlowContext();
    context.setActionName(actionCode);
    context.setBillType(billType);
    context.setBillVos(aggVOs);
    Logger.debug("开始调用动作脚本 ActionName[" + actionCode + "] BillType[" + billType
        + "]...");

    ICloudScriptPFlowService service =
        CommonUtil.getCloudScriptPFlowService();

    Object[] result = service.exeScriptPFlow(context);

    Logger.debug("调用动作脚本 ActionName[" + actionCode + "] BillType[" + billType
        + "]结束");

    String wheresql =
        "pk_bankcredit" + "='"
            + ((BankcreditBillVO) result[0]).getPrimaryKey() + "'";
    Collection<BankcreditBillVO> bills =
        CommonUtil.getMDPersistenceQueryService().queryBillOfVOByCond(
            BankcreditBillVO.class, wheresql, true, false);
    return bills.toArray(new BankcreditBillVO[0])[0];
  }
  
  /*
	 * 删除脚本
	 */
	private Object deleteVO(BankcreditBillVO... aggVOs)
		      throws BusinessException {

		    if ((aggVOs == null) || (aggVOs.length == 0)) {
		      return null;
		    }

		    String actionCode = BankcreditHVOConst.CONST_ACTION_DELETE;
		    String billType = BankcreditHVOConst.CONST_BILLTYPE_COST;

		    CloudPFlowContext context = new CloudPFlowContext();
		    context.setActionName(actionCode);
		    context.setBillType(billType);
		    context.setBillVos(aggVOs);
		    Logger.debug("开始调用动作脚本 ActionName[" + actionCode + "] BillType[" + billType
		        + "]...");

		    ICloudScriptPFlowService service = CommonUtil.getCloudScriptPFlowService();
		    Object[] result = service.exeScriptPFlow(context);

		    Logger.debug(
		        "调用动作脚本 ActionName[" + actionCode + "] BillType[" + billType + "]结束");

		    return result;
	}

  /**
   * 判断新增或修改
   *
   * @param vo
   */
  private void doBefore(BankcreditBillVO vo) {
    String parentPk = vo.getPrimaryKey();
    // 根据是否有主键信息判断是新增保存还是修改保存
    if ((parentPk != null) && !"".equals(parentPk)) {
      // 设置单据默认值
      CircularlyAccessibleValueObject[] allchildren = vo.getAllChildrenVO();
      if (allchildren != null) {
        for (CircularlyAccessibleValueObject obj : allchildren) {
          obj.setAttributeValue("pk_bankcredit", parentPk);
        }
      }
    }
  }
  
  /**
   * 判断组织月份
   * @return
 * @throws BusinessException 
   */
  private void judgmentmonth(BankcreditBillVO vo) throws BusinessException {
	  
	  if (vo.getParentVO().getMonth()<1 || vo.getParentVO().getMonth()>12) {
			throw new BusinessException("月份只允许填写1~12的数字！");
	  }
	  
	  String month = String.valueOf(vo.getParentVO().getMonth());
	  String org = vo.getParentVO().getPk_org();
	  String bankcredit = vo.getParentVO().getPk_bankcredit();
	  String sql = "";
	  
	  if(bankcredit != null) {
		  sql = "month = '"+String.valueOf(month)+"' and pk_org = '"+org+"' and pk_bankcredit != '"+bankcredit+"' and dr = '0' and def1 = '"+year+"'";
	  }else {
		  sql = "month = '"+String.valueOf(month)+"' and pk_org = '"+org+"' and dr = '0' and def1 = '"+year+"'";
	  }
		
	  BankcreditBillVO oldBankcreditBillVO = getVO(sql);
		
	  if (oldBankcreditBillVO==null) {
		  return;
	  }else {
		  deleteVO(oldBankcreditBillVO);
		  deletePk = oldBankcreditBillVO.getParentVO().getPk_bankcredit();
	  }
  }
  
  /**
   * 表头表体单位一致校验
   * @param vo
   * @throws DAOException
   */
  public void uniform(BankcreditBillVO vo) throws DAOException {
	
	  String headorg = vo.getParentVO().getPk_org();
	  String bodyorg = "";
	  
	  BankcreditBVO[] bvos = (BankcreditBVO[])vo.getChildrenVO();
	  
	  for (BankcreditBVO bankcreditBVO : bvos) {
		  
		  bodyorg = bankcreditBVO.getUnit();
		
		  if (!bodyorg.equals(headorg)) {
			throw new DAOException("表头一级组织与表体一级单位必须一致！");
		  }
		  
	  }
	  
  }
  
  /*
   * 校验单位映射是否正确
   */
  public void judgment(BankcreditBillVO bankcreditBillVO) throws DAOException {
		
	String unit = "";
	String credit = "";
	String unitfather = "";
	String creditunitfather = "";
	BaseDAO dao = new BaseDAO();
	
	BankcreditBVO[] childrenVO = (BankcreditBVO[]) bankcreditBillVO.getChildrenVO();
	
	String father = SqlQuery("pk_financeorg", "org_financeorg", "code", "10", dao, "一级财务组织编码10");
	
	String org = bankcreditBillVO.getParentVO().getPk_org();
	String orgfather = SqlQuery("pk_fatherorg", "org_orgs", "pk_org", org, dao, "组织父单位");
	String orgname = SqlQuery("name", "org_orgs", "pk_org", org, dao, "一级组织");
	
	if (orgfather!=null && !orgfather.equals(father)) {
		throw new DAOException("一级组织'"+orgname+"'不是一级组织，请检查数据的合法性！");
	}else if (orgname.equals("全局")) {
		throw new DAOException("一级组织不能为空，如是'全局'请修改！");
	}
		
	for (int i = 0; i < childrenVO.length; i++) {
			
		BankcreditBVO bvo = childrenVO[i];
			
		unit = bvo.getUnit();
		credit = bvo.getCreditunit();
		
		String unitName = SqlQuery("name", "org_financeorg", "pk_financeorg", unit, dao, "一级单位");
		String creditName = SqlQuery("name", "org_financeorg", "pk_financeorg", credit, dao, "授信单位");
		
		unitfather = SqlQuery("pk_fatherorg", "org_orgs", "pk_org", unit, dao, "一级父单位");
		
		if (unitfather!=null && !unitfather.equals(father)) {
			throw new DAOException("一级单位'"+unitName+"'不是一级组织，请检查数据的合法性！");
		}else if (credit.equals(father)&&!unit.equals(credit)) {
			throw new DAOException("授信单位'"+creditName+"'不是一级单位'"+unitName+"'的子公司，不能保存！");
		}else if(unit.equals(credit)) {
			return;
		}
		
		creditunitfather = SqlQuery("pk_fatherorg", "org_orgs", "pk_org", credit, dao, "授信父单位");
		
		for (int k = 0; k < 10; k++) {
			
			if (creditunitfather.equals(unit)) {
				break;
			}else if (creditunitfather.equals(father)) {
				throw new DAOException("授信单位'"+creditName+"'不是一级单位'"+unitName+"'的子公司，不能保存！");
			}else{
				creditunitfather=SqlQuery("pk_fatherorg", "org_orgs", "pk_org", creditunitfather, dao, "授信父单位");
			}
			
			
		}
	}
  }
  
  /*
	 * SQL查询方法
	 */
	private String SqlQuery(String out,String table,String lock,String key,BaseDAO dao, String remark) throws DAOException {
		
		if (key==null || key.equals("")) {
			return null;
		}
		
		String sql = "select "+out+" from "+table+" where "+lock+" = '"+key+"' and dr = '0'";
		
		Object object = dao.executeQuery(sql, new ColumnProcessor());
		
		if (remark.equals("一级父单位") && object == null) {
			return null;
		}else if (remark.equals("组织父单位") && object == null) {
			return null;
		}else if (object == null) {
			throw new DAOException("根据"+remark+"值"+key+"在数据库中未查到相应的数据，请检查！");
		}
		
		return object.toString();
	}
	
	/*
	 * VO查询方法
	 */
	@SuppressWarnings("unchecked")
	private BankcreditBillVO getVO(String sql) throws MetaDataException {
		
		IMDPersistenceQueryService qryService = NCLocator.getInstance().lookup(IMDPersistenceQueryService.class);
		
		ArrayList<BankcreditBillVO> AggVOs = (ArrayList<BankcreditBillVO>) qryService.queryBillOfVOByCond(BankcreditBillVO.class,sql,true,false);
		
		if (AggVOs.size()==0) {
			return null;
		}else {
			return AggVOs.get(0);
		}
	}
}
