package nccloud.zjnytwo_cd.zjnytwo_cd.bankloanhvo.action;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;

import nc.bs.dao.BaseDAO;
import nc.bs.dao.DAOException;
import nc.bs.framework.common.NCLocator;
import nc.bs.logging.Logger;
import nc.jdbc.framework.processor.ColumnProcessor;
import nc.md.model.MetaDataException;
import nc.md.persist.framework.IMDPersistenceQueryService;
import nc.vo.pub.BusinessException;
import nc.vo.pub.lang.UFDate;
import nc.vo.pub.lang.UFDouble;
import nc.vo.zjnytwo_cd.bankloan.BankloanBVO;
import nc.vo.zjnytwo_cd.bankloan.BankloanBillVO;
import nc.vo.zjnytwo_cd.bankloan.BankloanHVO;
import nccloud.framework.core.exception.ExceptionUtils;
import nccloud.framework.core.io.WebFile;
import nccloud.framework.web.action.itf.ICommonAction;
import nccloud.framework.web.container.IRequest;
import nccloud.framework.web.container.SessionContext;
import nccloud.pubitf.riart.pflow.CloudPFlowContext;
import nccloud.pubitf.riart.pflow.ICloudScriptPFlowService;
import nccloud.vo.zjnytwo_cd.zjnytwo_cd.BankloanHVOConst;
import nccloud.zjnytwo_cd.zjnytwo_cd.bankloanhvo.zjnytwo_cd.util.CommonUtil;
import nccloud.zjnytwo_cd.zjnytwo_cd.tool.Excel;

/**
 * 银行贷款填报导入
 * @author LP
 *
 */
public class BankloanHVOImportAction implements ICommonAction {

	public String group;
	public String org;
	public String org_v;
	public int month;
	public String year;
	
	public BaseDAO dao;
	public IMDPersistenceQueryService qryService;

	@Override
	public Object doAction(IRequest paramIRequest) {
		
		WebFile[] readFiles = paramIRequest.readFiles();
		
		WebFile file = readFiles[0];

		try {
			
			dao = new BaseDAO();
			qryService = NCLocator.getInstance().lookup(IMDPersistenceQueryService.class);
			
			JSONArray jsonArray = Excel.readXLSX(file);//解析Excel
			
			BankloanBillVO bankloanBillVO = setBankloanBillVO(jsonArray);
			
			saveBankloanBillVO(bankloanBillVO);
			
		} catch (Exception e) {
			Logger.error(e);
		    ExceptionUtils.wrapException(e);
		}
		
		return null;
	}
	
	/*
	 * 保存VO
	 */
	private void saveBankloanBillVO(BankloanBillVO bankloanBillVO) throws BusinessException {
		
		uniform(bankloanBillVO);
		
		judgment(bankloanBillVO);
		
		BankloanBillVO bVo = showBankloanBillVOs();
		
		if (bVo!=null) {
			deleteVO(bVo);//删除VO
		}
		saveVO(bankloanBillVO);
	}
	
	/**
	   * 表头表体单位一致校验
	   * @param vo
	   * @throws DAOException
	   */
	  public void uniform(BankloanBillVO vo) throws DAOException {
		
		  String headorg = vo.getParentVO().getPk_org();
		  String bodyorg = "";
		  
		  BankloanBVO[] bvos = (BankloanBVO[])vo.getChildrenVO();
		  
		  for (BankloanBVO bankloanBVO : bvos) {
			  
			  bodyorg = bankloanBVO.getUnit();
			
			  if (!bodyorg.equals(headorg)) {
				throw new DAOException("表头一级组织与表体一级单位必须一致！");
			  }
			  
		  }
		  
	  }
	
	/*
	   * 校验单位映射是否正确
	   */
	  public void judgment(BankloanBillVO bankloanBillVO) throws DAOException {
			
		String unit = "";
		String useunit = "";
		String unitfather = "";
		String useunitfather = "";
		
		BankloanBVO[] childrenVO = (BankloanBVO[]) bankloanBillVO.getChildrenVO();
		
		String father = SqlQuery("pk_financeorg", "org_financeorg", "code", "10", "一级财务组织编码10");
		
		String org = bankloanBillVO.getParentVO().getPk_org();
		String orgfather = SqlQuery("pk_fatherorg", "org_orgs", "pk_org", org, "组织父单位");
		String orgname = SqlQuery("name", "org_orgs", "pk_org", org, "一级组织");
		
		if (orgfather!=null && !orgfather.equals(father)) {
			throw new DAOException("一级组织'"+orgname+"'不是一级组织，请检查数据的合法性！");
		}else if (orgname.equals("全局")) {
			throw new DAOException("一级组织不能为空，如是'全局'请修改！");
		}
			
		for (int i = 0; i < childrenVO.length; i++) {
				
			BankloanBVO bvo = childrenVO[i];
				
			unit = bvo.getUnit();
			useunit = bvo.getUseunit();
			
			String unitName = SqlQuery("name", "org_financeorg", "pk_financeorg", unit, "一级单位");
			String useunitName = SqlQuery("name", "org_financeorg", "pk_financeorg", useunit, "使用单位");
			
			unitfather = SqlQuery("pk_fatherorg", "org_orgs", "pk_org", unit, "一级父单位");
			
			if (unitfather!=null && !unitfather.equals(father)) {
				throw new DAOException("一级单位'"+unitName+"'不是一级组织，请检查数据的合法性！");
			}else if (useunit.equals(father)&&!unit.equals(useunit)) {
				throw new DAOException("使用单位'"+useunitName+"'不是一级单位'"+unitName+"'的子公司，不能保存！");
			}else if(unit.equals(useunit)) {
				return;
			}
			
			useunitfather = SqlQuery("pk_fatherorg", "org_orgs", "pk_org", useunit, "使用父单位");
			
			for (int k = 0; k < 10; k++) {
				
				if (useunitfather.equals(unit)) {
					break;
				}else if (useunitfather.equals(father)) {
					throw new DAOException("使用单位'"+useunitName+"'不是一级单位'"+unitName+"'的子公司，不能保存！");
				}else{
					useunitfather=SqlQuery("pk_fatherorg", "org_orgs", "pk_org", useunitfather, "使用父单位");
				}
				
				
			}
		}
	  }
	
	/*
	 * 查找VO
	 */
	private BankloanBillVO showBankloanBillVOs() throws MetaDataException {
		
        String sql = "month = '"+String.valueOf(month)+"' and pk_org = '"+org+"' and dr = '0' and def1 = '"+year+"'";
		
		BankloanBillVO oldBankloanBillVO = getVO(sql);
		
		if (oldBankloanBillVO==null) {
			return null;
		}
		
		return oldBankloanBillVO;
	}
	
	/*
	 * 组合BankcreditBillVO
	 */
	private BankloanBillVO setBankloanBillVO(JSONArray jsonArray) throws Exception {
		
		BankloanBillVO bankloanBillVO = new BankloanBillVO();
		
		JSONArray headArray = jsonArray.getJSONArray(0);//表头
		JSONArray bodyArray = jsonArray.getJSONArray(1);//表体
		
		BankloanHVO headHvo = sethead(headArray);
		BankloanBVO[] setbody = setbody(bodyArray);
		
		bankloanBillVO.setParentVO(headHvo);
		bankloanBillVO.setChildrenVO(setbody);
		
		return bankloanBillVO;
	}
	
	/*
	 * 导入表头
	 */
	@SuppressWarnings("static-access")
	private BankloanHVO sethead(JSONArray headArray) throws Exception {
		
		BankloanHVO bankloantHVO = new BankloanHVO();
		JSONObject headObject = headArray.getJSONObject(0);
		
		String enablestate = headObject.getString("启用");
		
		if (headArray.size()>1) {
			throw new Exception("表头不允许有多列！请检查数据的合法性或Excel表格是否符合规范！");
		}
		
		if (enablestate==null) {
			throw new Exception("启用为必填项，请检查！");
		}
		
		if (headObject.getString("组织")==null) {
			throw new Exception("组织为必填项，请检查！");
		}
		
		if(enablestate.equals("Y")) {
			bankloantHVO.setEnablestate(2);
		}else if(enablestate.equals("N")) {
			bankloantHVO.setEnablestate(3);
		}else {
			throw new Exception("启用只允许填写'N'和'Y'，请检查！");
		}
		
		group = SqlQuery("pk_group", "org_group", "name", headObject.getString("集团"),"集团");
		org = SqlQuery("pk_financeorg", "org_financeorg", "name", headObject.getString("组织"),"组织");
		org_v = SqlQuery("pk_vid", "org_financeorg_v", "name", headObject.getString("组织"),"组织");
		
		bankloantHVO.setPk_group(group);
		bankloantHVO.setPk_org(org);
		bankloantHVO.setPk_org_v(org_v);
		
		month = Integer.valueOf(headObject.getString("月份"));
		
		if (month<1 || month>12) {
			throw new BusinessException("月份只允许填写1~12的数字！");
		}
		
		bankloantHVO.setMonth(month);//月份
		
		Date date = new Date();
		SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd"); 
		String Inputdate = formatter.format(date);
		bankloantHVO.setInputdate(new UFDate(Inputdate));//导入日期
		bankloantHVO.setDbilldate(new UFDate(Inputdate));//单据日期
		
		String cuserid = SessionContext.getInstance().getClientInfo().getUserid();
		
		bankloantHVO.setInputman(cuserid);
		bankloantHVO.setCode(headObject.getString("编码"));
		bankloantHVO.setName(headObject.getString("名称"));
		bankloantHVO.setVnote(headObject.getString("备注"));
		
		if(headObject.getString("年份").equals("")) {
			SimpleDateFormat SimpleDate = new SimpleDateFormat("yyyy");
	        year = SimpleDate.format(date);
	        bankloantHVO.setDef1(year);
		}else {
			if (this.isNumber(headObject.getString("年份"))) {
				year = headObject.getString("年份");
				bankloantHVO.setDef1(year);
			}
		}
		
		return bankloantHVO;
	}
	
	/**
	 * 判断年份
	 * @param str
	 * @return
	 * @throws Exception
	 */
	private static boolean isNumber(String str) throws Exception{
        String reg = "^[0-9]+(.[0-9]+)?$";
        if(str.matches(reg)) {
        	if (str.length()>4 || str.length()<4) {
				throw new Exception("请输入正确的年份！");
			}else {
				return true;
			}
        }else {
        	throw new Exception("年份只允许填入数字，请检查！");
		}
    }
	
	/*
	 * 导入表体
	 */
	private BankloanBVO[] setbody(JSONArray bodyArray) throws DAOException {
		
		BankloanBVO[] bvos = new BankloanBVO[bodyArray.size()];
		
		for(int i = 0 ; i < bodyArray.size() ; i++ ) {
			
			JSONObject bodyobject = bodyArray.getJSONObject(i);
			
			BankloanBVO bankloanBVO = new BankloanBVO();
			
			bankloanBVO.setPk_group(group);
			bankloanBVO.setPk_org(org);
			bankloanBVO.setPk_org_v(org_v);
			bankloanBVO.setCrowno(bodyobject.getString("行号"));
			bankloanBVO.setUnit(SqlQuery("pk_financeorg", "org_financeorg", "name", bodyobject.getString("一级单位"),"一级单位"));
			bankloanBVO.setUseunit(SqlQuery("pk_financeorg", "org_financeorg", "name", bodyobject.getString("使用单位"),"使用单位"));
			bankloanBVO.setBankname(bodyobject.getString("银行名称"));
			bankloanBVO.setLoanamount(new UFDouble(bodyobject.getString("借款金额")));
			bankloanBVO.setRepayamount(new UFDouble(bodyobject.getString("已还款金额")));
			bankloanBVO.setLoanbalance(new UFDouble(bodyobject.getString("借款余额")));
			bankloanBVO.setLoantype(bodyobject.getString("借款种类"));
			bankloanBVO.setStartdate(new UFDate(bodyobject.getString("借款起始日")));
			bankloanBVO.setEnddate(new UFDate(bodyobject.getString("借款到期日")));
			bankloanBVO.setTerm(bodyobject.getString("期限"));
			bankloanBVO.setRates(bodyobject.getString("实际利率"));
			bankloanBVO.setSettletype(bodyobject.getString("解决类型"));
			bankloanBVO.setBackedunit(bodyobject.getString("担保单位"));
			bankloanBVO.setDocumentcode(bodyobject.getString("单据编号"));
			bankloanBVO.setVbnote(bodyobject.getString("行备注"));
			
			bvos[i] = bankloanBVO;
		}
		
		return bvos;
	}
	
	/*
	 * VO查询方法
	 */
	@SuppressWarnings("unchecked")
	private BankloanBillVO getVO(String sql) throws MetaDataException {
		ArrayList<BankloanBillVO> AggVOs = (ArrayList<BankloanBillVO>) qryService.queryBillOfVOByCond(BankloanBillVO.class,sql,true,false);
		
		if (AggVOs.size()==0) {
			return null;
		}
		
		return AggVOs.get(0);
	}
	
	/*
	 * SQL查询方法
	 */
	private String SqlQuery(String out,String table,String lock,String key,String remark) throws DAOException {
		
		if (key==null || key.equals("")) {
			return null;
		}
		
		String sql = "select "+out+" from "+table+" where "+lock+" = '"+key+"' and dr = '0'";
		
		Object object = dao.executeQuery(sql, new ColumnProcessor());
		
		if (remark.equals("一级父单位") && object == null) {
			return null;
		}else if (remark.equals("组织父单位") && object == null) {
			return null;
		}else if (object == null) {
			throw new DAOException("根据"+remark+"值"+key+"在数据库中未查到相应的数据，请检查！");
		}
		
		return object.toString();
	}
	
	/*
	 * 保存脚本
	 */
	@SuppressWarnings("unchecked")
	private BankloanBillVO saveVO(BankloanBillVO... aggVOs)
		      throws BusinessException {

		    String actionCode = BankloanHVOConst.CONST_ACTION_SAVEBASE;
		    String billType = BankloanHVOConst.CONST_BILLTYPE_COST;

		    CloudPFlowContext context = new CloudPFlowContext();
		    context.setActionName(actionCode);
		    context.setBillType(billType);
		    context.setBillVos(aggVOs);
		    Logger.debug("开始调用动作脚本 ActionName[" + actionCode + "] BillType[" + billType
		        + "]...");

		    ICloudScriptPFlowService service =
		        CommonUtil.getCloudScriptPFlowService();

		    Object[] result = service.exeScriptPFlow(context);

		    Logger.debug("调用动作脚本 ActionName[" + actionCode + "] BillType[" + billType
		        + "]结束");

		    String wheresql =
		        "pk_bankloan" + "='"
		            + ((BankloanBillVO) result[0]).getPrimaryKey() + "'";
		    Collection<BankloanBillVO> bills =
		        CommonUtil.getMDPersistenceQueryService().queryBillOfVOByCond(
		            BankloanBillVO.class, wheresql, true, false);
		    return bills.toArray(new BankloanBillVO[0])[0];
	}
	
	/*
	 * 删除脚本
	 */
	private Object deleteVO(BankloanBillVO... aggVOs)
		      throws BusinessException {

		    if ((aggVOs == null) || (aggVOs.length == 0)) {
		      return null;
		    }

		    String actionCode = BankloanHVOConst.CONST_ACTION_DELETE;
		    String billType = BankloanHVOConst.CONST_BILLTYPE_COST;

		    CloudPFlowContext context = new CloudPFlowContext();
		    context.setActionName(actionCode);
		    context.setBillType(billType);
		    context.setBillVos(aggVOs);
		    Logger.debug("开始调用动作脚本 ActionName[" + actionCode + "] BillType[" + billType
		        + "]...");

		    ICloudScriptPFlowService service = CommonUtil.getCloudScriptPFlowService();
		    Object[] result = service.exeScriptPFlow(context);

		    Logger.debug(
		        "调用动作脚本 ActionName[" + actionCode + "] BillType[" + billType + "]结束");

		    return result;
		  }

}
