package nccloud.zjnytwo_cd.zjnytwo_cd.tool;

import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.poi.hssf.usermodel.HSSFDateUtil;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFCellStyle;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;

import nccloud.framework.core.io.WebFile;

public class Excel {

	/**
	 * Excel导出
	 * @param
	 * @param title
	 * @param values
	 * @return
	 */
    public static XSSFWorkbook outExcel(String sheetName,String []title,String [][]values) throws UnsupportedEncodingException {

    	XSSFWorkbook wb = new XSSFWorkbook();
        XSSFSheet sheet = wb.createSheet(sheetName);
        XSSFRow row = sheet.createRow(0);
        XSSFCellStyle style = wb.createCellStyle();
        style.setAlignment(HorizontalAlignment.CENTER);
        XSSFCell cell = null;
        
        for(int i=0;i<title.length;i++) {

            cell = row.createCell(i);
            cell.setCellValue(new String(title[i].getBytes(), "UTF-8"));
            cell.setCellStyle(style);
            
        }
        for(int i=0;i<values.length;i++) {
        	
            row = sheet.createRow(i + 1);
            
            for(int j=0;j<values[i].length;j++) {

                row.createCell(j).setCellValue(values[i][j]);
                
            }
        }
        return wb;
    }
    
    /**
	 * 解析Excel
	 * @param
	 * @throws IOException
	 */
	public static JSONArray readXLSX(WebFile file) throws IOException {
		// 获取excel文件的io流
		InputStream is = file.getInputStream();
		// 创建Workbook工作薄对象，表示整个excel
		XSSFWorkbook book = new XSSFWorkbook(is);
		Iterator<Sheet> items = book.iterator();
		Map<String, Object> map = new HashMap<>();
		int i = 0;
		JSONArray jsonArray = new JSONArray();
		
		while (items.hasNext()) {
			
			Sheet sheet = items.next();
			
			if (i==0) {
				map.put("head", read(sheet, book));
				jsonArray.add(JSONArray.parseArray(new String(JSONObject.toJSONString(map.get("head")))));
			}else {
				map.put("body", read(sheet, book));
				jsonArray.add(JSONArray.parseArray(new String(JSONObject.toJSONString(map.get("body")))));
			}
			i++;
		}
		return jsonArray;
	}
	
	/**
	 * 解析Excel
	 * @param
	 * @throws IOException
	 */
	public static JSONArray readXLSX(XSSFWorkbook book) throws IOException {
		Iterator<Sheet> items = book.iterator();
		Map<String, Object> map = new HashMap<>();
		while (items.hasNext()) {
			Sheet sheet = items.next();
			map.put("sheet", read(sheet, book));
		}
		return JSONArray.parseArray(JSONObject.toJSONString(map.get("sheet")));
	}
	
	public static List<Map<String, Object>> read(Sheet sheet, Workbook book) throws IOException {
		int rowStart = sheet.getFirstRowNum(); // 首行下标
		int rowEnd = sheet.getLastRowNum(); // 尾行下标
		// 如果首行与尾行相同，表明只有一行，直接返回空数组
		if (rowStart == rowEnd) {
//				book.close();
			return new ArrayList<>();
		}
		// 获取第一行JSON对象键
		Row firstRow = sheet.getRow(rowStart);
		int cellStart = firstRow.getFirstCellNum();
		int cellEnd = firstRow.getLastCellNum();
		Map<Integer, String> keyMap = new HashMap<>();
		for (int j = cellStart; j < cellEnd; j++) {
			String a= String.valueOf(firstRow.getCell(j));
			if (a==null||a.equals("")){
				cellEnd=j;
				continue;
			}
			keyMap.put(j, getValue(firstRow.getCell(j), rowStart, j, book, true));
		}
		// 获取每行JSON对象的值
		List<Map<String, Object>> mapList = new ArrayList<>();
		for (int i = rowStart + 1; i <= rowEnd; i++) {
			Row eachRow = sheet.getRow(i);
			if (eachRow != null) {
				Map<String, Object> map = new HashMap<>();
				StringBuffer sb = new StringBuffer();
				for (int k = cellStart; k < cellEnd; k++) {

					String val = getValue(eachRow.getCell(k), i, k, book, false);
					sb.append(val); // 所有数据添加到里面，用于判断该行是否为空
					map.put(keyMap.get(k), val);
				}

				if (sb.toString().length() > 0) {
					mapList.add(map);
				}
			}
		}
//			book.close();
		return mapList;
	}
	
	public static String getValue(Cell cell, int rowNum, int index, Workbook book, boolean isKey) throws IOException {

		// 空白或空
		if (cell == null || cell.getCellType() == CellType.BLANK) {
			if (isKey) {
//					book.close();
				throw new NullPointerException(String.format("the key on row %s index %s is null ", ++rowNum, ++index));
			} else {
				return "";
			}
		}

		// 0. 数字 类型
		if (cell.getCellType() == CellType.NUMERIC) {
			if (HSSFDateUtil.isCellDateFormatted(cell)) {
				Date date = cell.getDateCellValue();
				DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
				return df.format(date);
			}
//			String val = String.valueOf(Math.round(cell.getNumericCellValue()));
			String val = cell.getNumericCellValue() + "";
			val = val.toUpperCase();
			if (val.contains("E")) {
				val = new BigDecimal(val+"").toString();
			}
			return val;
		}

		// 1. String类型
		if (cell.getCellType() == CellType.STRING) {
			String val = cell.getStringCellValue();
			if (val == null || val.trim().length() == 0) {
//					if (book != null) {
//						book.close();
//					}
				return "";
			}
			return val.trim();
		}

		// 2. 公式 CELL_TYPE_FORMULA
		if (cell.getCellType() == CellType.FORMULA) {
			return cell.getStringCellValue();
		}

		// 4. 布尔值 CELL_TYPE_BOOLEAN
		if (cell.getCellType() == CellType.BOOLEAN) {
			return cell.getBooleanCellValue() + "";
		}

		// 5. 错误 CELL_TYPE_ERROR
		return "";
	}
	
	/**
	* 处理导入小数点
	*/
	@SuppressWarnings("deprecation")
	public static String numOfImport(Cell cell) {
	    String value = cell.toString();
	    CellType i = cell.getCellType();
	    if (i.getCode() == 1) {//字符串类型
	        return value;
	    } else {
	        String[] str = value.split("\\.");
	        if (str.length > 1) {
	            String str1 = str[1];
	            int m = Integer.parseInt(str1);
	            if (m == 0) {
	                return str[0];
	            } else {
	                return value;
	            }
	        }else{
	            return value;
	        }
	    }


	}

}
